// TAffineTransformOp.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;

class TAffineTransformOp extends JFrame {
    DisplayPanel displayPanel;
    JComboBox scaleXval, scaleYval, shearXval, shearYval;
    String[] scaleValues = {"0.10", "0.25", "0.50", "0.75", "1.00",
                            "1.25", "1.50", "1.75", "2.00"};
    String[] shearValues = {"0.00", "0.25", "0.50", "0.75", "1.00"};

    public TAffineTransformOp() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TAffineTransformOp");
        Container container = getContentPane();

        // 2. Define and add a panel to display images.
        displayPanel = new DisplayPanel();
        container.add(displayPanel);

        // 3. Create a control panel and add the panel to the 
        // the main frame.
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 4, 5, 5));
        panel.setBorder(new TitledBorder(
            "Select a Value from a Combo Box..."));

        scaleXval = new JComboBox(scaleValues);
        scaleXval.setSelectedItem("1.00");
        scaleXval.addActionListener(new ComboBoxListener());
        scaleYval = new JComboBox(scaleValues);
        scaleYval.setSelectedItem("1.00");
        scaleYval.addActionListener(new ComboBoxListener());

        shearXval = new JComboBox(shearValues);
        shearXval.setSelectedItem("0.00");
        shearXval.addActionListener(new ComboBoxListener());
        shearYval = new JComboBox(shearValues);
        shearYval.setSelectedItem("0.00");
        shearYval.addActionListener(new ComboBoxListener());

        panel.add(new JLabel("Scale X value:"));
        panel.add(scaleXval);
        panel.add(new JLabel("Scale Y value:"));
        panel.add(scaleYval);
        panel.add(new JLabel("Shear X value:"));
        panel.add(shearXval);
        panel.add(new JLabel("Shear Y value:"));
        panel.add(shearYval); 

        container.add(BorderLayout.SOUTH, panel);

        // 4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        // Assign suitable size. 10 is for the control panel
        setSize(displayPanel.getWidth(), displayPanel.getHeight() + 10);
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. The main method...
    public static void main(String arg[]) {
        new TAffineTransformOp();
    }

    
    // 7. Action listener to handle the events fired by combo boxes.
    class ComboBoxListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JComboBox temp = (JComboBox) e.getSource();

            if (temp == scaleXval) {
                displayPanel.scalex = Double.parseDouble(
                                      (String)temp.getSelectedItem());
                displayPanel.applyValue(true, false);
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (temp == scaleYval) {
                displayPanel.scaley = Double.parseDouble(
                                      (String)temp.getSelectedItem());
                displayPanel.applyValue(true, false);
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (temp == shearXval) {
                displayPanel.shearx = Double.parseDouble(
                                      (String)temp.getSelectedItem());
                displayPanel.applyValue(false, true);
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (temp == shearYval) {
                displayPanel.sheary = Double.parseDouble(
                                      (String)temp.getSelectedItem());
                displayPanel.applyValue(false, true);
                displayPanel.applyFilter();
                displayPanel.repaint();
            } 
        }
    }
}

// 8. Panel to display images
class DisplayPanel extends JPanel {
    Image displayImage;
    BufferedImage biSrc, biDest; // Source and Destination
    BufferedImage bi; // For reference purpose
    Graphics2D big;

    AffineTransform transform;
    double scalex = 1.0;
    double scaley = 1.0;
    double shearx = 1.0;
    double sheary = 1.0;

    // Constructor
    DisplayPanel() {
        setBackground(Color.black);  // panel background color
        loadImage();
        setSize(displayImage.getWidth(this),
                displayImage.getWidth(this));  // panel width and height

        createBufferedImages();

        // Create the affine transform object.
        transform = new AffineTransform(); // Identity transform
    }

    // This method loads the image.
    public void loadImage() {
        // Create an image object using the specified file
        displayImage = Toolkit.getDefaultToolkit().getImage("images/Tiger.jpg");
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(displayImage, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading.");
        }
        // If the image has an unknown width, the image is not created
        // by using the suggested file. Therefore, exit the program.  
        if (displayImage.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the suggested image "
                            + "(*.jpg) file in the same directory.***");
            System.exit(0);
        }
    }

    // This method creates the buffered images: source and destination.
    public void createBufferedImages() {
        // NOTE: Do not use createImage() method here. As the
        // component paint() method is not at called, the createImage()
        // returns a null object.
        biSrc = new BufferedImage(displayImage.getWidth(this),
                                  displayImage.getHeight(this),
                                  BufferedImage.TYPE_INT_RGB);

        // Draw the texture image into the memory buffer.
        big = biSrc.createGraphics();
        big.drawImage(displayImage, 0, 0, this);

        // Intially display this image
        bi = biSrc;

        // Create the destination image
        biDest = new BufferedImage(displayImage.getWidth(this),
                                   displayImage.getHeight(this),
                                   BufferedImage.TYPE_INT_RGB);
    }

    // This method applies the scaling and shearing values to
    // the affine transforms.
    public void applyValue(boolean scale, boolean shear) {
        if (scale) {       
            transform.setToScale(scalex, scaley);
            scale = false;
        }
        else if (shear) {
            transform.setToShear(shearx, sheary);
            shear = false;
        }
    }

    // This method applies the affine transform filter to the
    // source image.
    public void applyFilter() {
        AffineTransformOp op = new AffineTransformOp(
                                   transform, null);
        Graphics2D biDestG2D = biDest.createGraphics();
        biDestG2D.clearRect(0, 0, biDest.getWidth(this),
                                  biDest.getHeight(this));
        op.filter(biSrc, biDest);
        bi = biDest;
    }

    // Resets the panel to the initial image.
    public void reset() {
        big.setColor(Color.black);
        big.clearRect(0, 0, bi.getWidth(this), bi.getHeight(this));
        big.drawImage(displayImage, 0, 0, this);
    }

    // The update method
    public void update(Graphics g) {
        g.clearRect(0, 0, getWidth(), getHeight());
        paintComponent(g);
    }

    // The paintComponent method
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        
        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        // Draw the buffered Image on to the screen
        g2D.drawImage(bi, 0,0, this);
    }
}
