// TBandCombineOp.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class TBandCombineOp extends JFrame {
    DisplayPanel displayPanel;
    JButton redBandButton, greenBandButton, blueBandButton,
            inverseBandButton, middleBandButton,
            resetButton;
    
    public TBandCombineOp() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TBandCombineOp");
        Container container = getContentPane();

        // 2. Define and add a panel that displays images.
        displayPanel = new DisplayPanel();
        container.add(displayPanel);

        // 3. Create a panel to add the control buttons. Add the
        // panel to the main frame.
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(3, 2));
        panel.setBorder(new TitledBorder(
            "Click a Button to Perform the Associated Operation and Reset..."));

        redBandButton = new JButton("Show Red Band");
        redBandButton.addActionListener(new ButtonListener());
        greenBandButton = new JButton("Show Green Band");
        greenBandButton.addActionListener(new ButtonListener());
        blueBandButton = new JButton("Show Blue Band");
        blueBandButton.addActionListener(new ButtonListener());
        inverseBandButton = new JButton("Invert All Bands");
        inverseBandButton.addActionListener(new ButtonListener());
        middleBandButton = new JButton("Average Each Band");
        middleBandButton.addActionListener(new ButtonListener());

        resetButton = new JButton("Reset");
        resetButton.addActionListener(new ButtonListener());
        
        // Add the buttons to the panel. 
        panel.add(redBandButton);
        panel.add(blueBandButton);
        panel.add(greenBandButton);
        panel.add(inverseBandButton);
        panel.add(middleBandButton);
        panel.add(resetButton);
        // Add the panel at the bottom portion of the main frame.
        container.add(BorderLayout.SOUTH, panel);

        // 4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        // Assign suitable size.
        setSize(displayPanel.getWidth(), displayPanel.getHeight());
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. The main method...
    public static void main(String arg[]) {
        new TBandCombineOp();
    }

    
    // 7. Action listener to handle the events fired by buttons.
    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JButton button = (JButton) e.getSource();

            // If the red band button is clicked...
            if (button.equals(redBandButton)) {
                displayPanel.bandCombine(DisplayPanel.RED_BAND_MATRIX);
                displayPanel.repaint();
            }
            // If the green band button is clicked...
            else if (button.equals(greenBandButton)) {
                displayPanel.bandCombine(DisplayPanel.GREEN_BAND_MATRIX);
                displayPanel.repaint();
            }
            // If the blue band button is clicked...
            else if (button.equals(blueBandButton)) {
                displayPanel.bandCombine(DisplayPanel.BLUE_BAND_MATRIX);
                displayPanel.repaint();
            }
            // If the button to invert all bands is clicked...
            else if (button.equals(inverseBandButton)) {
                displayPanel.bandCombine(DisplayPanel.INVERSE_BAND_MATRIX);
                displayPanel.repaint();
            }
            // If the button to consider half of each band intensity is clicked...
            else if (button.equals(middleBandButton)) {
                displayPanel.bandCombine(DisplayPanel.AVERAGE_BAND_MATRIX);
                displayPanel.repaint();
            }
            // If the reset button is clicked...
            else if (button.equals(resetButton)) {
                displayPanel.reset();
                displayPanel.repaint();
            }
        }
    } 
}

// 8. Definition of the display panel.
class DisplayPanel extends JPanel {
    // Matrix that shows only the red band
    static final float RED_BAND_MATRIX[][] = {{1.0f, 0.0f, 0.0f},
                                              {0.0f, 0.0f, 0.0f},
                                              {0.0f, 0.0f, 0.0f}};
    // Matrix that shows only the green band
    static final float GREEN_BAND_MATRIX[][] = {{0.0f, 0.0f, 0.0f},
                                                {0.0f, 1.0f, 0.0f},
                                                {0.0f, 0.0f, 0.0f}};
    // Matrix that shows only the blue band
    static final float BLUE_BAND_MATRIX[][] = {{0.0f, 0.0f, 0.0f},
                                               {0.0f, 0.0f, 0.0f},
                                               {0.0f, 0.0f, 1.0f}};
    // Matrix that inverts all the bands
    // That is, this matrix produces the nagative of the image.
    static final float INVERSE_BAND_MATRIX[][] = {{-1.0f, 0.0f, 0.0f},
                                                  {0.0f, -1.0f, 0.0f},
                                                  {0.0f, 0.0f, -1.0f}};

    // Matrix that reduces the intensities of all bands
    static final float AVERAGE_BAND_MATRIX[][] = {{0.5f, 0.0f, 0.0f},
                                                 {0.0f, 0.5f, 0.0f},
                                                 {0.0f, 0.0f, 0.5f}};

    // For loading the given image 
    Image displayImage;
    
    // The source and destination images for the display purpose.
    BufferedImage biSrc;
    BufferedImage biDest; 

    // The source and destination rasters for the filtering purpose 
    Raster srcRaster;
    WritableRaster dstRaster;

    BufferedImage bi; // Only an additional reference.
    Graphics2D big;

    // 9. Constructor
    DisplayPanel() {
        setBackground(Color.black);  // panel background color
        loadImage();
        setSize(displayImage.getWidth(this),
                displayImage.getWidth(this));  // panel width and height

        createBufferedImages();
        bi = biSrc;
    }

    // 10. This method loads the specified image.
    public void loadImage() {
        // Create an image object using the specified file
        displayImage = Toolkit.getDefaultToolkit().getImage("images/Flamingo.jpg");
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(displayImage, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading.");
        }

        // If the image has an unknown width, the image is not created
        // by using the suggested file. Therefore exit the program.  
        if (displayImage.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the image "
                            + "(*.jpg) file in the same directory.***");
            System.exit(0);
        }
    }

    // 11. This method creates the source and destination buffers
    public void createBufferedImages() {
        // NOTE: Do not use createImage() method here. As the
        // component paint() method is not at called, the createImage()
        // returns a null object.
        biSrc = new BufferedImage(displayImage.getWidth(this),
                                  displayImage.getHeight(this),
                                  BufferedImage.TYPE_INT_RGB);

        // Draw the texture image into the memory buffer.
        big = biSrc.createGraphics();
        big.drawImage(displayImage, 0, 0, this);

        // Retrieve the raster of this image
        srcRaster = biSrc.getRaster();

        // Create a buffer for the destination image.
        // NOTE: You can not use the source buffer.
        biDest = new BufferedImage(displayImage.getWidth(this),
                                  displayImage.getHeight(this),
                                  BufferedImage.TYPE_INT_RGB);

        dstRaster = (WritableRaster) biDest.getRaster();
    }

    // 12. This method creates a band combine operation filter using the
    // specified matrix, and then filters the source image.
    public void bandCombine(float[][] bandCombineMatrix) {
        BandCombineOp bandCombineOp = new BandCombineOp(bandCombineMatrix,
                                                        null);
        bandCombineOp.filter(srcRaster, dstRaster);
        bi = biDest;
    }

    // 13. Reset the display panel
    public void reset() {
        big.setColor(Color.black);
        big.clearRect(0, 0, bi.getWidth(this), bi.getHeight(this));
        big.drawImage(displayImage, 0, 0, this);
        bi = biSrc;
    }

    // 14. The update method...
    public void update(Graphics g) {
        g.clearRect(0, 0, getWidth(), getHeight());
        paintComponent(g);
    }

    // 15. The paintComponent method...
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        // Draw the buffered Image on to the screen
        g2D.drawImage(bi, 0,0, this);
    }
}
