// TColorConvertOp.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.color.*;
import java.awt.image.*;

class TColorConvertOp extends JFrame {
    DisplayPanel displayPanel;
    JButton grayButton, resetButton;
    
    public TColorConvertOp() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TColorConvertOp");
        Container container = getContentPane();

        // 2. Create a display panel and add it to the main frame.
        displayPanel = new DisplayPanel();
        container.add(displayPanel);

        // 3. Create a control panel and add it to the main frame
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(1, 2));
        panel.setBorder(new TitledBorder(
            "Click the Gray Scale Button to Create Gray Scale Image..."));

        grayButton = new JButton("Gray Scale");
        grayButton.addActionListener(new ButtonListener());
        resetButton = new JButton("Reset");
        resetButton.addActionListener(new ButtonListener());

        panel.add(grayButton);
        panel.add(resetButton);

        container.add(BorderLayout.SOUTH, panel);

        // 4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        // Assign suitable size. 15 is for the control panel
        setSize(displayPanel.getWidth(), displayPanel.getHeight() + 15);
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. The main method...
    public static void main(String arg[]) {
        new TColorConvertOp();
    }

    // 7. Action listener to handle the event fired by buttons.
    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JButton button = (JButton) e.getSource();

            if (button.equals(grayButton)) {
                displayPanel.grayOut();
                displayPanel.repaint();
            }
            else if (button.equals(resetButton)) {
                displayPanel.reset();
                displayPanel.repaint();
            }
        }
    } 
}

// 8. Definition of the diaplay panel.
class DisplayPanel extends JPanel {
    Image displayImage;
    BufferedImage bi;
    Graphics2D big;

    // Constructor
    DisplayPanel() {
        setBackground(Color.black);  // panel background color
        loadImage();
        setSize(displayImage.getWidth(this),
                displayImage.getWidth(this));  // panel width and height

        createBufferedImage();
    }

    // This method loads the specified image
    public void loadImage() {
        // Create an image object using the specified file
        displayImage = Toolkit.getDefaultToolkit().getImage("images/car.jpg");
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(displayImage, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading.");
        }
        // If the image has an unknown width, the image is not created
        // by using the suggested file. Therefore, exit the program.  
        if (displayImage.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the suggested image "
                            + "(*.jpg) file in the same directory.***");
            System.exit(0);
        }
    }

    // This method creates a buffered image object.
    public void createBufferedImage() {
        // NOTE: Do not use createImage() method here. As the
        // component paint() method is not at called, the createImage()
        // returns a null object.
        bi = new BufferedImage(displayImage.getWidth(this),
                               displayImage.getHeight(this),
                               BufferedImage.TYPE_INT_RGB);

        // Draw the texture image into the memory buffer.
        big = bi.createGraphics();
        big.drawImage(displayImage, 0, 0, this);
    }

    // This method creates a color conversion filter and applies
    // the filtering operation on the buffered image.
    public void grayOut() {
        ColorConvertOp colorConvert = new ColorConvertOp(
            ColorSpace.getInstance(ColorSpace.CS_GRAY), null);
        colorConvert.filter(bi, bi);
    }

    // Reset the display panel
    public void reset() {
        big.setColor(Color.black);
        big.clearRect(0, 0, bi.getWidth(this), bi.getHeight(this));
        big.drawImage(displayImage, 0, 0, this);
    }

    // The update method
    public void update(Graphics g) {
        g.clearRect(0, 0, getWidth(), getHeight());
        paintComponent(g);
    }

    // The paintComponent method
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        // Draw the buffered Image on to the screen
        g2D.drawImage(bi, 0,0, this);
    }
}
