// TConvolveOp.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class TConvolveOp extends JFrame {
    DisplayPanel displayPanel;
    JButton sharpenButton, blurringButton,
            edButton, resetButton;
    
    public TConvolveOp() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TConvolveOp");
        Container container = getContentPane();

        // 2. Define and add a panel that displays images.
        displayPanel = new DisplayPanel();
        container.add(displayPanel);

        // 3. Create a panel to add the control buttons. Add the
        // panel to the main frame.
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 2));
        panel.setBorder(new TitledBorder(
            "Click a Button to Perform the Associated Operation and Reset..."));

        sharpenButton = new JButton("Sharpen");
        sharpenButton.addActionListener(new ButtonListener());
        blurringButton = new JButton("Blur");
        blurringButton.addActionListener(new ButtonListener());
        edButton = new JButton("Edge Detect");
        edButton.addActionListener(new ButtonListener());
        resetButton = new JButton("Reset");
        resetButton.addActionListener(new ButtonListener());

        panel.add(sharpenButton);
        panel.add(blurringButton);
        panel.add(edButton);
        panel.add(resetButton);

        container.add(BorderLayout.SOUTH, panel);

        // 4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        // Assign suitable size. 10 is for the control panel
        setSize(displayPanel.getWidth(), displayPanel.getHeight() + 10);
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. The main method...
    public static void main(String arg[]) {
        new TConvolveOp();
    }

    
    // 7. Action listener to handle the events fired by buttons.
    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JButton button = (JButton) e.getSource();

            if (button.equals(sharpenButton)) {
                displayPanel.sharpen();
                displayPanel.repaint();
            }
            else if (button.equals(blurringButton)) {
                displayPanel.blur();
                displayPanel.repaint();
            }
            else if (button.equals(edButton)) {
                displayPanel.edgeDetect();
                displayPanel.repaint();
            }
            else if (button.equals(resetButton)) {
                displayPanel.reset();
                displayPanel.repaint();
            }
        }
    } 
}

// 8. Definition of the display panel.
class DisplayPanel extends JPanel {
    Image displayImage;
    BufferedImage biSrc;
    BufferedImage biDest; // Destination image is mandetory.
    BufferedImage bi; // Only an additional reference.
    Graphics2D big;

    // Constructor
    DisplayPanel() {
        setBackground(Color.black);  // panel background color
        loadImage();
        setSize(displayImage.getWidth(this),
                displayImage.getWidth(this));  // panel width and height

        createBufferedImages();
        bi = biSrc;
    }

    // This method loads the specified image.
    public void loadImage() {
        // Create an image object using the specified file
        displayImage = Toolkit.getDefaultToolkit().getImage("images/car.jpg");
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(displayImage, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading.");
        }
        // If the image has an unknown width, the image is not created
        // by using the suggested file. Therefore, exit the program.  
        if (displayImage.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the suggested image "
                            + "(*.jpg) file in the same directory.***");
            System.exit(0);
        }
    }

    // This method creates the source and destination buffers
    public void createBufferedImages() {
        // NOTE: Do not use createImage() method here. As the
        // component paint() method is not at called, the createImage()
        // returns a null object.
        biSrc = new BufferedImage(displayImage.getWidth(this),
                                  displayImage.getHeight(this),
                                  BufferedImage.TYPE_INT_RGB);

        // Draw the texture image into the memory buffer.
        big = biSrc.createGraphics();
        big.drawImage(displayImage, 0, 0, this);

        // Create a buffer for the destination image.
        // NOTE: You can not use the source buffer.
        biDest = new BufferedImage(displayImage.getWidth(this),
                                  displayImage.getHeight(this),
                                  BufferedImage.TYPE_INT_RGB);
    }

    // This method creates a convolution mask to sharpen the image,
    // a convolution filter and then applies the filter on the
    // source image.
    public void sharpen() {
        // Define the kernel data which is basically the
        // Sharpening mask.
        float data[] = {-1.0f, -1.0f, -1.0f,
                        -1.0f,  9.0f, -1.0f,
                        -1.0f, -1.0f, -1.0f};
        Kernel kernel = new Kernel(3, 3, data);
        ConvolveOp convolve = new ConvolveOp(kernel,
                                             ConvolveOp.EDGE_NO_OP,
                                             null);
        convolve.filter(biSrc, biDest);
        bi = biDest;
    }

    // This method creates a convolution mask to blur the image,
    // a convolution filter and then applies the filter on the
    // source image.

    public void blur() {
        float data[] = {0.0625f, 0.125f, 0.0625f,
                        0.125f,  0.25f,  0.125f,
                        0.0625f, 0.125f, 0.0625f};
        Kernel kernel = new Kernel(3, 3, data);
        ConvolveOp convolve = new ConvolveOp(kernel,
                                             ConvolveOp.EDGE_NO_OP,
                                             null);
        convolve.filter(biSrc, biDest);
        bi = biDest;
    }
    // This method creates convolution mask to detect the edges in the image,
    // a convolution filter and then applies the filter on the
    // source image.

    public void edgeDetect() {
        // Prewitt's  horizontal convolution mask
        float data[] = {1.0f, 0.0f, -1.0f,
                        1.0f, 0.0f, -1.0f,
                        1.0f, 0.0f, -1.0f};   

        Kernel kernel = new Kernel(3, 3, data);
        ConvolveOp convolve = new ConvolveOp(kernel,
                                             ConvolveOp.EDGE_NO_OP,
                                             null);
        convolve.filter(biSrc, biDest);

        bi = biDest;
    }

    // Reset the display panel
    public void reset() {
        big.setColor(Color.black);
        big.clearRect(0, 0, bi.getWidth(this), bi.getHeight(this));
        big.drawImage(displayImage, 0, 0, this);
        bi = biSrc;
    }

    // The update method
    public void update(Graphics g) {
        g.clearRect(0, 0, getWidth(), getHeight());
        paintComponent(g);
    }

    // The paintComponent method
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        // Draw the buffered Image on to the screen
        g2D.drawImage(bi, 0,0, this);
    }
}
