// TPrinting1.java

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.awt.print.*;

class TPrinting1 extends JFrame {
    DrawingCanvas canvas;
    JButton setUpButton, printButton, cancelButton;

    public TPrinting1() {
        // 1. Assign a name to the frame and retrieve its content pane
        super("TPrinting1");
        Container container = getContentPane();

        // 2. Add the canvas 
        canvas = new DrawingCanvas();
        container.add(canvas);

        // 3. Create the buttons panel
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(1,3));

        setUpButton = new JButton("Page Setup");
        ButtonListener buttonListener = new ButtonListener();
        setUpButton.addActionListener(buttonListener);
        panel.add(setUpButton);
        printButton = new JButton("Print");
        printButton.addActionListener(buttonListener);
        panel.add(printButton);
        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(buttonListener);
        panel.add(cancelButton);
        // Add the panel at the bottom of the frame
        container.add(BorderLayout.SOUTH, panel);

        // 4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        setSize(400, 275); // Frame size
        show(); // Display the frame
    }

    // 5. Button listener to handle action events
    class ButtonListener implements ActionListener {
        PrinterJob printJob;
        PageFormat pageFormat;
        PrintableCanvas printableCanvas;

        ButtonListener() {
            // 6. Create a printer job
            printJob = PrinterJob.getPrinterJob();

            // 7. Obtain a default page format object
            pageFormat = printJob.defaultPage();
        }

        public void actionPerformed(ActionEvent e) {
            JButton tempButton = (JButton) e.getSource();

            if (tempButton.equals(setUpButton)) {
                pageFormat = printJob.pageDialog(pageFormat);
                printJob.validatePage(pageFormat);
            }
            else if (tempButton.equals(printButton)) {
                printableCanvas = new PrintableCanvas(pageFormat);
                printJob.setPrintable(printableCanvas);

                boolean ok = printJob.printDialog();
                if (ok) {
                    try {
                        printJob.print();
                    } catch (Exception pe) {
                        System.out.println("Printing Exception Occured!");
                        pe.printStackTrace();
                    }
                }
            }
            else if (tempButton.equals(cancelButton)) {
                printJob.cancel();
            }
        }
    }

    // 9. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 10. The main method...
    public static void main(String arg[]) {
        new TPrinting1();
    }
}

// 11. The display canvas 
class DrawingCanvas extends JPanel {
    Font font;
    FontMetrics fontMetrics;
    int w, h;

    // 12. Constructor
    DrawingCanvas() {
        setBackground(Color.white);
        setSize(400, 275);

        // Obtain the canvas dimensions to display graphics
        // elements appropriately.
        w = this.getWidth();
        h = this.getHeight();

        // Font used for text, and its font metrics
        font = new Font("Dialog", Font.BOLD, 50);
        fontMetrics = getFontMetrics(font);
    }

    // 13. Overridden paint component method 
    public void paintComponent(Graphics g) {
        super.paintComponent(g); // important for proper behaviour of
                // of containers, especially for assigning background color.

        // Set up the 2D graphics context
        Graphics2D g2D = (Graphics2D) g;

        // Paint the contents of this canvas
        paintContent(g2D, w, h);
    }

    // 14. Write graphics content to the printer graphics context using the
    // dimensions of the imageable area.
    public void paintContent(Graphics2D g2D, int w, int h) {
        // Assign the font object
        g2D.setFont(font);

        // Create a gradient paint object
        GradientPaint gp = new GradientPaint(
                   (float)(0.5*(w-fontMetrics.stringWidth("Java 2D"))),
                   (float)(0.5*h+fontMetrics.getHeight()-10.0),
                   Color.blue,
                   (float)(0.5*w+0.5*fontMetrics.stringWidth("Java 2D")),
                   (float)(0.5*h+fontMetrics.getHeight()-10.0),
                   Color.red);
        g2D.setPaint(gp);  // Assign the gradient paint to the graphics context

        // Draw the string Java 2D
        g2D.drawString("Java 2D",
                       (float)(0.5*(w-fontMetrics.stringWidth("Java 2D"))),
                       (float)(0.5*h-1.25*fontMetrics.getHeight()));

        // Draw the string "Printing Demo"
        AffineTransform at = new AffineTransform();
        at.setToTranslation(
            0.5*(w-fontMetrics.stringWidth("Printing Demo")),
            0.5*h-0.25*fontMetrics.getHeight());
        at.shear(-0.5, 0.0);
        FontRenderContext frc = new FontRenderContext(at, false, false);
        TextLayout tl = new TextLayout("Printing Demo", font, frc);
        Shape outline = tl.getOutline(null);
        BasicStroke wideStroke = new BasicStroke(2.0f);
        g2D.setStroke(wideStroke);
        g2D.draw(outline);
    }
}

// 15. Encapsulation for the printable object
class PrintableCanvas implements Printable {
    DrawingCanvas canvas;
    PageFormat pageFormat;

    public PrintableCanvas(PageFormat pf) {
        pageFormat = pf; 
    }

    // 16. Implementation method from the Printable interface
    public int print(Graphics g, PageFormat pageFormat, int pageIndex)
                   throws PrinterException {
        if (pageIndex >= 1) {
            return Printable.NO_SUCH_PAGE;
        }

        // Setup the 2D graphics context for printing
        Graphics2D g2D = (Graphics2D) g;

        canvas = new DrawingCanvas();

        // Use the current format for printing
        canvas.paintContent(g2D,
                            (int) pageFormat.getImageableWidth(),
                            (int) pageFormat.getImageableHeight());

        // Notify on successful printing of the page
        return Printable.PAGE_EXISTS;
    }
}



